# -*- coding: utf-8 -*-

from odoo import api, models, _
from odoo.exceptions import UserError, ValidationError
from odoo.tools import email_domain_extract


class MailServer(models.Model):
    _inherit = 'ir.mail_server'

    @api.model
    def _match_from_filter(self, email_from, from_filter):
        """Specialized comparison for odoo.com and mail.odoo.com, as we are authorized
        to send emails from "mail.odoo.com" on mail servers that have an "odoo.com" from_filter.

        Without this, emails sent with the automatic bounce address "bounce@mail.odoo.com" are
        stripped of that bounce address due to the from filter, and the bounces go directly
        to the user's inbox instead of being processed (and turning the "chatter envelope" red)
        """
        from_domain = email_domain_extract(email_from)
        if from_domain == "mail.odoo.com" and from_filter == "odoo.com":
            return True
        return super()._match_from_filter(email_from, from_filter)


class MailComposeMessage(models.TransientModel):
    _inherit = 'mail.compose.message'

    def send_mail(self, auto_commit=False):
        for composer in self:
            if composer.mass_mailing_id.mail_server_id and not composer.mail_server_id:
                composer.mail_server_id = composer.mass_mailing_id.mail_server_id.id
        super(MailComposeMessage, self).send_mail(auto_commit=auto_commit)


class MailThread(models.AbstractModel):
    _inherit = 'mail.thread'

    def _notify_record_by_email(self, *args, **kwargs):
        # Always defer emails to avoid concurrent updates
        kwargs['force_send'] = False
        kwargs['send_after_commit'] = True
        return super()._notify_record_by_email(*args, **kwargs)


class MassMailing(models.Model):
    _inherit = 'mailing.mailing'

    @api.constrains('mailing_domain')
    def _check_no_blacklist(self):
        for rec in self:
            if rec.mailing_domain:
                if 'is_blacklisted' in rec.mailing_domain:
                    raise ValidationError(
                        _('Please remove any criterion on the "Blacklist" field, it makes everything '
                          'very slow, and is unnecessary! Blacklisted records are automatically '
                          'excluded from mass mailings as of Odoo 12.'))


    def _get_default_mailing_domain(self):
        mailing_domain = super()._get_default_mailing_domain()
        # filter out tuple that contains `is_blacklisted`
        # c.f. https://github.com/odoo/internal/commit/14a53234d02b03ec696580711ed506d1f853112e
        mailing_domain = [d for d in mailing_domain if d != ('is_blacklisted', '=', False)]
        return mailing_domain


class MailTemplate(models.Model):
    _inherit = 'mail.template'

    def write(self, values):
        self._restrict_modification()
        return super(MailTemplate, self).write(values)

    def _restrict_modification(self):
        """ We have decided to restrict the modification of `mail.template` that have an xmlid
            assigned to an installed module to selected groups.

            This is to avoid the modification of important templates. We had an issue with the
            invoice one that had the "To" parameter modified to a static value and sent some
            invoices to the wrong recipient.
        """
        self.ensure_one()
        if self.env.user._is_admin() or self.env.su or self.env.user.has_group('openerp_enterprise.group_oe_saas'):
            return

        xml_ids = self._get_external_ids()
        xml_id_modules = set([xml_id.split('.')[0]
                             for xml_ids_values in xml_ids.values()
                             for xml_id in xml_ids_values])
        unrestricted_modules = {'__import__', '__export__'}
        if xml_id_modules - unrestricted_modules:
            raise UserError(
                "Sorry, you are not allowed to modify built-in mail templates. "
                "This operation is allowed for the groups:\n\t- Administration/Settings\n\t- OE_SAAS")
